/* FILE: error.c
 *
 * Copyright (c) 2005 James A. Strother
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "osivs-devel.h"


/**********************************************************
 * The following function returns a string which describes
 * the error code contained in the integer 'e'. It returns
 * NULL if the error code is unrecognized.
 **********************************************************/
const char*
osivs_strerr(
  osivs_int e)
{
   const char* s;
 
   switch (e)
   {
   case OSIVS_EMEM:
      s = "out of memory";
      break;
   case OSIVS_ETRIES:
      s = "failed to achive minimum spacing";
      break;
   case OSIVS_ETREE:
      s = "quadtree was corrupted";
      break;
   case OSIVS_EFFMT:
      s = "filter output has invalid format";
      break;
   case OSIVS_EFORK:
      s = "process fork failed";
      break;
   case OSIVS_EFILT:
      s = "filter exited with error code";
      break;
   case OSIVS_EFLESS:
      s = "filter produced too few results";
      break;
   case OSIVS_EFMORE:
      s = "filter produced too many results";
      break;
   case OSIVS_ETEMP:
      s = "unable to create temporary file";
      break;
   case OSIVS_EIBITS:
      s = "image depth must be less 16";
      break;
   case OSIVS_EOUTF:
      s = "unable to open output file";
      break;
   case OSIVS_EXSIZE:
      s = "image width must be greater than zero";
      break;
   case OSIVS_EYSIZE:
      s = "image height must be greater than zero";
      break;
   case OSIVS_EZSIZE:
      s = "light sheet thickness must be positive";
      break;
   case OSIVS_EXEDGE:
      s = "image x margin must be positive";
      break;
   case OSIVS_EYEDGE:
      s = "image y margin must be positive";
      break;
   case OSIVS_EZEDGE:
      s = "image z margin must be positive";
      break;
   case OSIVS_ESPACE:
      s = "particle spacing must be positive";
      break;
   case OSIVS_EDENS:
      s = "particle density must be positive";
      break;
   case OSIVS_ESMIN:
      s = "min particle size must be positive";
      break;
   case OSIVS_ESMAX:
      s = "max particle size must be positive";
      break;
   case OSIVS_ESCMP:
      s = "min particle size must less than max";
      break;
   case OSIVS_ESAVG:
      s = "particle size average must be positive";
      break;
   case OSIVS_ESDEV:
      s = "particle size stddev must be positive";
      break;
   case OSIVS_ESBND:
      s = "average particle size must be within min-max range";
      break;
   case OSIVS_EBMIN:
      s = "min particle intensity must be positive";
      break;
   case OSIVS_EBMAX:
      s = "max particle intensity must be positive";
      break;
   case OSIVS_EBCMP:
      s = "min particle intensity must be less than max";
      break;
   case OSIVS_EBAVG:
      s = "particle intensity average must be positive";
      break;
   case OSIVS_EBDEV:
      s = "particle intensity stddev must be positive";
      break;
   case OSIVS_EBBND:
      s = "average particle intensity must be within min-max range";
      break;
   default: return NULL;
   }

   return s;
}

