/* FILE: scat.c
 *
 * Copyright (c) 2005 James A. Strother
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "osivs-devel.h"


/**********************************************************
 * The following function fills in some of the particle
 * attributes.  By this point each of the particles should
 * have a valid sz, bt1, and bt2.  However, these need to
 * be scaled and randomized according to input parameters
 **********************************************************/
osivs_int
osivs_scat(
   osivs_opts_t* p,
   osivs_pts_t* pts)
{
   int i;

   for (i = 0; i < pts->num; i++) {
      osivs_real sz;
      osivs_real bt1;
      osivs_real bt2;

      /* re-sample to get contrained size */
      do {
         sz = osivs_randn() * p->dot_size_dev
            + pts->sz[i] * p->dot_size_avg;
      }  while (
         sz < p->dot_size_min || sz > p->dot_size_max
      );

      /* re-sample to get contrained brite */
      do {
         bt2 = bt1 = osivs_randn();

         bt1 = bt1 * p->dot_brite_dev +
            pts->bt1[i] * p->dot_brite_avg;
         bt2 = bt2 * p->dot_brite_dev +
            pts->bt2[i] * p->dot_brite_avg;
      } while (
         bt1 < p->dot_brite_min || bt1 > p->dot_brite_max ||
         bt2 < p->dot_brite_min || bt2 > p->dot_brite_max
      );

      /* place found values in array */
      pts->sz[i] = sz;
      pts->bt1[i] = bt1;
      pts->bt2[i] = bt2;
   }

   return 0;
}


/**********************************************************
 * The following function translates the particle by
 * the velocity given in the u, v, and w field.
 **********************************************************/
osivs_int
osivs_disp(
   osivs_opts_t* p,
   osivs_pts_t* pts)
{
   int i;

   for (i = 0; i < pts->num; i++) {
      pts->x[i] += pts->u[i];
      pts->y[i] += pts->v[i];
      pts->z[i] += pts->w[i];
   }

   return 0;
}

