/* FILE: extrema.c 
 * 
 * Copyright (c) 2002 James A. Strother 
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 * 
 */

#include "osivx-devel.h"


/*********************************************************
 * The following macro defines a function that finds
 * the minimum value of the passed array of unsigned
 * integers.  This setup behaves as a sort of a
 * "poor man's" version of templates.
 *********************************************************/
#define DEFINE_FIND_MIN_UNSIGN(TYPE)                  \
osivx_int                                             \
osivx_find_min_##TYPE (                               \
   osivx_##TYPE##_t* array,                           \
   osivx_int len)                                     \
{                                                     \
   register osivx_int i = 0;                          \
   register osivx_int best = 0;                       \
   osivx_##TYPE##_t lastval = *(array + len - 1);     \
                                                      \
   *(array + len - 1) = 0;                            \
   while (i < len - 1) {                              \
      best = i++;                                     \
      while (*(array + i) > *(array + best)) {        \
         i++;                                         \
      }                                               \
   }                                                  \
   *(array + len - 1) = lastval;                      \
                                                      \
   if (*(array + best) >= lastval) {                  \
      return len - 1;                                 \
   } else {                                           \
      return best;                                    \
   }                                                  \
}


/*********************************************************
 * The following macro defines a function that finds
 * the maximum value of the passed of array of unsigned
 * integers.  This setup behaves as a sort of a
 * "poor man's" version of templates.
 *********************************************************/
#define DEFINE_FIND_MAX_UNSIGN(TYPE)                  \
osivx_int                                             \
osivx_find_max_##TYPE (                               \
   osivx_##TYPE##_t* array,                           \
   osivx_int len)                                     \
{                                                     \
   register osivx_int i = 0;                          \
   register osivx_int best = 0;                       \
   osivx_##TYPE##_t lastval = *(array + len - 1);     \
                                                      \
   *(array + len - 1) = ~((osivx_##TYPE##_t)0);       \
   while (i < len - 1) {                              \
      best = i++;                                     \
      while (*(array + i) < *(array + best)) {        \
         i++;                                         \
      }                                               \
   }                                                  \
   *(array + len - 1) = lastval;                      \
                                                      \
   if (*(array + best) <= lastval) {                  \
      return len - 1;                                 \
   } else {                                           \
      return best;                                    \
   }                                                  \
}


/*********************************************************
 * The following macro defines a function that finds
 * the minimum value of the passed array of signed
 * integers.  This setup behaves as a sort of a
 * "poor man's" version of templates.
 *********************************************************/
#define DEFINE_FIND_MIN_SIGNED(TYPE)                  \
osivx_int                                             \
osivx_find_min_##TYPE (                               \
   osivx_##TYPE##_t* array,                           \
   osivx_int len)                                     \
{                                                     \
   register osivx_int i = 0;                          \
   register osivx_int best = 0;                       \
   osivx_##TYPE##_t lastval = *(array + len - 1);     \
                                                      \
   *(array + len - 1) = (osivx_##TYPE##_t)            \
     (((osivx_##TYPE##_t)1) <<                        \
      (8*sizeof(osivx_##TYPE##_t)-1)                  \
     );                                               \
   while (i < len - 1) {                              \
      best = i++;                                     \
      while (*(array + i) > *(array + best)) {        \
         i++;                                         \
      }                                               \
   }                                                  \
   *(array + len - 1) = lastval;                      \
                                                      \
   if (*(array + best) >= lastval) {                  \
      return len - 1;                                 \
   } else {                                           \
      return best;                                    \
   }                                                  \
}


/*********************************************************
 * The following macro defines a function that finds
 * the maximum value of the passed of array of signed
 * integers.  This setup behaves as a sort of a
 * "poor man's" version of templates.
 *********************************************************/
#define DEFINE_FIND_MAX_SIGNED(TYPE)                  \
osivx_int                                             \
osivx_find_max_##TYPE (                               \
   osivx_##TYPE##_t* array,                           \
   osivx_int len)                                     \
{                                                     \
   register osivx_int i = 0;                          \
   register osivx_int best = 0;                       \
   osivx_##TYPE##_t lastval = *(array + len - 1);     \
                                                      \
   *(array + len - 1) = (osivx_##TYPE##_t)            \
     ~(((osivx_##TYPE##_t)1) <<                       \
      (8*sizeof(osivx_##TYPE##_t)-1)                  \
     );                                               \
   while (i < len - 1) {                              \
      best = i++;                                     \
      while (*(array + i) < *(array + best)) {        \
         i++;                                         \
      }                                               \
   }                                                  \
   *(array + len - 1) = lastval;                      \
                                                      \
   if (*(array + best) <= lastval) {                  \
      return len - 1;                                 \
   } else {                                           \
      return best;                                    \
   }                                                  \
}


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_imcp_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_UNSIGN(imcp)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_imsp_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_SIGNED(imsp)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_immp_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_UNSIGN(immp)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_imcw_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_UNSIGN(imcw)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_imsw_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_SIGNED(imsw)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_immw_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_UNSIGN(immw)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_imcf_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_UNSIGN(imcf)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_imsf_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_SIGNED(imsf)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of osivx_immf_t integers.  It returns
 * the index of the minimum value.
 *********************************************************/
DEFINE_FIND_MIN_UNSIGN(immf)


/*********************************************************
 * The following function find the minimum value in the
 * passed array of fftw_real floating point numbers.  The
 * array must be of the form generated by FFTW for an
 * xlen by ylen window, with an offset of yskip from start
 * of one row to the start of the next row.  It will only
 * return values that have a maximum x/y displacement of
 * xdisp/ydisp.   It returns the index of the minimum
 * value.
 *********************************************************/
osivx_int
osivx_find_min_fftw(
   fftw_real * array,
   osivx_int xlen,
   osivx_int ylen,
   osivx_int xdisp,
   osivx_int ydisp,
   osivx_int yskip)
{
   register osivx_int i, j;
   register osivx_int best = 0;

   /* perform fast search if possible */
   if (xdisp == xlen / 2 && ydisp == ylen / 2) {
      for (i = 0; i < ylen * yskip; i += yskip) {
         for (j = i; j < xlen + i; j++) {
            if (*(array + j) <= *(array + best)) {
               best = j;
            }
         }
      }
      return best;
   }

   /* otherwise look more carefully */
   for (i = 0; i <= ydisp; i++) {
      for (j = i * yskip; j <= xdisp + i * yskip; j++) {
         if (*(array + j) <= *(array + best)) {
            best = j;
         }
      }

      j = xlen - xdisp;
      for (j += i * yskip; j < xlen + i * yskip; j++) {
         if (*(array + j) <= *(array + best)) {
            best = j;
         }
      }
   }

   for (i = ylen - ydisp; i < ylen; i++) {
      for (j = i * yskip; j <= xdisp + i * yskip; j++) {
         if (*(array + j) <= *(array + best)) {
            best = j;
         }
      }

      j = xlen - xdisp;
      for (j += i * yskip; j < xlen + i * yskip; j++) {
         if (*(array + j) <= *(array + best)) {
            best = j;
         }
      }
   }

   return best;
}


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_imcp_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_UNSIGN(imcp)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_imsp_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_SIGNED(imsp)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_immp_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_UNSIGN(immp)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_imcw_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_UNSIGN(imcw)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_imsw_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_SIGNED(imsw)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_immw_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_UNSIGN(immw)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_imcf_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_UNSIGN(imcf)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_imsf_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_SIGNED(imsf)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of osivx_immf_t integers.  It returns
 * the index of the maximum value.
 *********************************************************/
DEFINE_FIND_MAX_UNSIGN(immf)


/*********************************************************
 * The following function find the maximum value in the
 * passed array of fftw_real floating point numbers.  The
 * array must be of the form generated by FFTW for an
 * xlen by ylen window, with an offset of yskip from start
 * of one row to the start of the next row.  It will only
 * return values that have a maximum x/y displacement of
 * xdisp/ydisp.   It returns the index of the maximum 
 * value.
 *********************************************************/
osivx_int
osivx_find_max_fftw(
   fftw_real * array,
   osivx_int xlen,
   osivx_int ylen,
   osivx_int xdisp,
   osivx_int ydisp,
   osivx_int yskip)
{
   register osivx_int i, j;
   register osivx_int best = 0;

   /* perform fast search if possible */
   if (xdisp == xlen / 2 && ydisp == ylen / 2) {
      for (i = 0; i < ylen * yskip; i += yskip) {
         for (j = i; j < xlen + i; j++) {
            if (*(array + j) >= *(array + best)) {
               best = j;
            }
         }
      }
      return best;
   }

   /* otherwise look more carefully */
   for (i = 0; i <= ydisp; i++) {
      for (j = i * yskip; j <= xdisp + i * yskip; j++) {
         if (*(array + j) >= *(array + best)) {
            best = j;
         }
      }
 
      j = xlen - xdisp;
      for (j += i * yskip; j < xlen + i * yskip; j++) {
         if (*(array + j) >= *(array + best)) {
            best = j;
         }
      }
   }
 
   for (i = ylen - ydisp; i < ylen; i++) {
      for (j = i * yskip; j <= xdisp + i * yskip; j++) {
         if (*(array + j) >= *(array + best)) {
            best = j;
         }
      }
 
      j = xlen - xdisp;
      for (j += i * yskip; j < xlen + i * yskip; j++) {
         if (*(array + j) >= *(array + best)) {
            best = j;
         }
      }
   }

   return best;
}

