/* FILE: alg1.c 
 *
 * Copyright (c) 2002-2006 James A. Strother 
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 * 
 */

#include "osivx-devel.h"


#include "osivx-devel.h"

#include <math.h>


/**********************************************************
 * The following macro defines a function for performing
 * minimum peak interpolation.
 **********************************************************/
#define DEFINE_INTERP_MIN_NO_EXPAN(TYPE,NAME)              \
void                                                       \
osivx_interp_min_##NAME(                                   \
   osivx_interp_t * interp,                                \
   TYPE * ind,                                             \
   osivx_int lft,                                          \
   osivx_int rht,                                          \
   osivx_int up,                                           \
   osivx_int dwn,                                          \
   osivx_real * x,                                         \
   osivx_real * y)                                         \
{                                                          \
   switch (interp->type) {                                 \
   case 0:                                                 \
      break;                                               \
   case 1:                                                 \
      osivx_interp_min1(                                   \
         *(ind + lft), *(ind), *(ind + rht), x);           \
      osivx_interp_min1(                                   \
         *(ind + up), *(ind), *(ind + dwn), y);            \
      break;                                               \
   case 2:                                                 \
      osivx_interp_min2(                                   \
         *(ind + lft), *(ind), *(ind + rht), x);           \
      osivx_interp_min2(                                   \
         *(ind + up), *(ind), *(ind + dwn), y);            \
      break;                                               \
   case 3:                                                 \
      osivx_interp_min3(                                   \
         *(ind + up + lft), *(ind + up), *(ind + up + rht), \
         *(ind + lft), *(ind), *(ind + rht),               \
         *(ind + dwn + lft), *(ind + dwn),                 \
         *(ind + dwn + rht), x, y);                        \
      break;                                               \
   case 4:                                                 \
      osivx_interp_min4(                                   \
         *(ind + lft), *(ind), *(ind + rht), x);           \
      osivx_interp_min4(                                   \
         *(ind + up), *(ind), *(ind + dwn), y);            \
      break;                                               \
   }                                                       \
}

#define DEFINE_INTERP_MIN_NO_SUFFIX(TYPE) \
  DEFINE_INTERP_MIN_NO_EXPAN(osivx_##TYPE, TYPE)

#define DEFINE_INTERP_MIN(TYPE) \
  DEFINE_INTERP_MIN_NO_EXPAN(osivx_##TYPE##_t, TYPE)


/**********************************************************
 * The following macro defines a function for performing
 * minimum peak interpolation.
 **********************************************************/
#define DEFINE_INTERP_MAX_NO_EXPAN(TYPE,NAME)              \
void                                                       \
osivx_interp_max_##NAME(                                   \
   osivx_interp_t * interp,                                \
   TYPE * ind,                                             \
   osivx_int lft,                                          \
   osivx_int rht,                                          \
   osivx_int up,                                           \
   osivx_int dwn,                                          \
   osivx_real * x,                                         \
   osivx_real * y)                                         \
{                                                          \
   switch (interp->type) {                                 \
   case 0:                                                 \
      break;                                               \
   case 1:                                                 \
      osivx_interp_max1(                                   \
         *(ind + lft), *(ind), *(ind + rht), x);           \
      osivx_interp_max1(                                   \
         *(ind + up), *(ind), *(ind + dwn), y);            \
      break;                                               \
   case 2:                                                 \
      osivx_interp_max2(                                   \
         *(ind + lft), *(ind), *(ind + rht), x);           \
      osivx_interp_max2(                                   \
         *(ind + up), *(ind), *(ind + dwn), y);            \
      break;                                               \
   case 3:                                                 \
      osivx_interp_max3(                                   \
         *(ind + up + lft), *(ind + up), *(ind + up + rht), \
         *(ind + lft), *(ind), *(ind + rht),               \
         *(ind + dwn + lft), *(ind + dwn),                 \
         *(ind + dwn + rht), x, y);                        \
      break;                                               \
   case 4:                                                 \
      osivx_interp_max4(                                   \
         *(ind + lft), *(ind), *(ind + rht), x);           \
      osivx_interp_max4(                                   \
         *(ind + up), *(ind), *(ind + dwn), y);            \
      break;                                               \
   }                                                       \
}

#define DEFINE_INTERP_MAX_NO_SUFFIX(TYPE) \
  DEFINE_INTERP_MAX_NO_EXPAN(osivx_##TYPE, TYPE)

#define DEFINE_INTERP_MAX(TYPE) \
  DEFINE_INTERP_MAX_NO_EXPAN(osivx_##TYPE##_t, TYPE)



/**********************************************************
 * The following function initializes the interpolation
 * engine.  It returns NULL on failure.
 **********************************************************/
osivx_interp_t *
osivx_interp_init(
   osivx_int type)
{
   osivx_interp_t *out;

   out = (osivx_interp_t *) osivx_malloc(sizeof(osivx_interp_t));
   if (out == NULL) {
      return NULL;
   }

   out->type = type;
   return out;
}


/**********************************************************
 * The following function frees an interpolation engine.
 * It returns zero on success and a non-zero error code
 * on failure.
 **********************************************************/
osivx_int
osivx_interp_free(
   osivx_interp_t * i)
{
   osivx_free(i);
   return OSIVX_EOKAY;
}


/**********************************************************
 * The following function returns OSIVX_INTERP_EIGHT
 * if the eight pixel neighboorhood is accessed,
 * OSIVX_INTERP_FOUR if the four pixel neighboorhood
 * is accessed, and OSIVX_INTERP_NONE if no pixels
 * are accessed.
 **********************************************************/
int
osivx_interp_usage(
   const osivx_interp_t* interp)
{
   switch (interp->type) {
   case 0: return OSIVX_INTERP_NONE;
   case 1: return OSIVX_INTERP_FOUR;
   case 2: return OSIVX_INTERP_FOUR;
   case 3: return OSIVX_INTERP_EIGHT;
   case 4: return OSIVX_INTERP_FOUR;
   }

   /* should never get here */
   return OSIVX_INTERP_NONE;
}


/**********************************************************
 * The following function finds the minimum of a
 * function using 3-point Gaussian interpolation.  This
 * function cannot generate errors.
 **********************************************************/
static void
osivx_interp_min1(
   osivx_real lftval,
   osivx_real midval,
   osivx_real rhtval,
   osivx_real * x)
{
   /* TODO: we have to flip the function for this to
    * work properly but how far should we offset?
    * Gui (2000) doesn't seem to present method.
    * The value 1.5 was determined empirically
    * and seems to work well.
    */

   osivx_real off = 1.5*(lftval + midval + rhtval);

   *x += 0.5 * log((off-lftval) / (off-rhtval)) /
       log( (off-lftval) * (off-rhtval) /
            ((off-midval) * (off-midval)));
}


/**********************************************************
 * The following function finds the minimum of a
 * function using parabolic interpolation.  This
 * function cannot generate errors.
 **********************************************************/
static void
osivx_interp_min2(
   osivx_real lftval,
   osivx_real midval,
   osivx_real rhtval,
   osivx_real * x)
{
   *x += 0.5 * (lftval - rhtval) /
     (lftval + rhtval - 2 * midval);
}


/**********************************************************
 * The following function finds the minimum of a
 * function using paraboloidal interpolation.  This
 * function cannot generate errors.
 **********************************************************/
static void
osivx_interp_min3(
   osivx_real v0,
   osivx_real v1,
   osivx_real v2,
   osivx_real v3,
   osivx_real v4,
   osivx_real v5,
   osivx_real v6,
   osivx_real v7,
   osivx_real v8,
   osivx_real * x,
   osivx_real * y)
{
   osivx_interp_min2(
      v0 + v3 + v6,
      v1 + v4 + v7,
      v2 + v5 + v8, x);
   osivx_interp_min2(
      v0 + v1 + v2,
      v3 + v4 + v5,
      v6 + v7 + v8, y);
}


/**********************************************************
 * The following function finds the minimum of a
 * function using centroid interpolation.  This
 * function cannot generate errors.
 **********************************************************/
static void
osivx_interp_min4(
   osivx_real lftval,
   osivx_real midval,
   osivx_real rhtval,
   osivx_real * x)
{
   *x += (lftval - rhtval) /
      (lftval + midval + rhtval);
}


/**********************************************************
 * The following function finds the maximum of a
 * function using 3-point Gaussian interpolation.  This
 * function cannot generate errors.
 **********************************************************/
static void
osivx_interp_max1(
   osivx_real lftval,
   osivx_real midval,
   osivx_real rhtval,
   osivx_real * x)
{
   *x += .5 * log(lftval / rhtval) /
       log(lftval * rhtval / (midval * midval));
}


/**********************************************************
 * The following function finds the maximum of a
 * function using parabolic interpolation.  This
 * function cannot generate errors.
 **********************************************************/
static void
osivx_interp_max2(
   osivx_real lftval,
   osivx_real midval,
   osivx_real rhtval,
   osivx_real * x)
{
   *x += .5 * (lftval - rhtval) /
      (lftval + rhtval - 2 * midval);
}


/**********************************************************
 * The following function finds the maximum of a
 * function using paraboloid interpolation.  This
 * function cannot generate errors.
 **********************************************************/
static void
osivx_interp_max3(
   osivx_real v0,
   osivx_real v1,
   osivx_real v2,
   osivx_real v3,
   osivx_real v4,
   osivx_real v5,
   osivx_real v6,
   osivx_real v7,
   osivx_real v8,
   osivx_real * x,
   osivx_real * y)
{
   osivx_interp_max2(
      v0 + v3 + v6,
      v1 + v4 + v7,
      v2 + v5 + v8, x);
   osivx_interp_max2(
      v0 + v1 + v2,
      v3 + v4 + v5,
      v6 + v7 + v8, y);
}


/**********************************************************
 * The following function finds the maximum of a
 * function using centroid interpolation.  This
 * function cannot generate errors.
 **********************************************************/
static void
osivx_interp_max4(
   osivx_real lftval,
   osivx_real midval,
   osivx_real rhtval,
   osivx_real * x)
{
   *x += (rhtval - lftval) /
      (lftval + midval + rhtval);
}


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imcp_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(imcp)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imsp_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(imsp)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_immp_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(immp)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imcw_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(imcw)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imsw_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(imsw)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_immw_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(immw)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imcf_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(imcf)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imsf_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(imsf)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_immf_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN(immf)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_real. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN_NO_SUFFIX(real)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type fftw_real. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MIN_NO_EXPAN(fftw_real, fftw)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imcp_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(imcp)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imsp_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(imsp)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_immp_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(immp)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imcw_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(imcw)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imsw_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(imsw)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_immw_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(immw)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imcf_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(imcf)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_imsf_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(imsf)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_immf_t. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX(immf)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type osivx_real. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX_NO_SUFFIX(real)


/**********************************************************
 * The following function performs interpolation around
 * the given point, given data of type fftw_real. This
 * function cannot generate errors.
 **********************************************************/
DEFINE_INTERP_MAX_NO_EXPAN(fftw_real, fftw)


