/* FILE: osivx-devel.h 
 * 
 * Copyright (c) 2003-2006 James A. Strother 
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 * 
 */ 

#ifndef __OSIVX_DEVEL_H__
#define __OSIVX_DEVEL_H__

#include "osivx.h"

#include "config.h"


/* 
 *  The following type definitions are for use within
 *  the library and development purposes only
 */

/* make threads chunk size */
#define OSIVX_THRD_CHUNK 100

/* make fftw type */ 
typedef double fftw_real; 


/* make osivx_grid_t types */
enum {
   OSIVX_GRID_RECT
};

typedef struct
{
   osivx_int type;

   union {
      struct {
         osivx_int xstart;
         osivx_int ystart;
         osivx_int xspacing;
         osivx_int yspacing;
         osivx_int xsize;
         osivx_int ysize;
         osivx_int wind_xsize;
         osivx_int wind_ysize;
         osivx_int wind_xsuper;
         osivx_int wind_ysuper;
      } rect;
   } u;
} osivx_grid_t;

typedef struct
{
   const osivx_grid_t* grid_ptr;

   union {
      struct {
         osivx_int pos;
      } rect;
   } u;
} osivx_grid_iter_t;


/* make osivx_interp_t type */
typedef struct
{
  osivx_int type;
} osivx_interp_t;


/* make osivx_movie_t type */
typedef struct
{
  osivx_int type;
  osivx_int xsize;
  osivx_int ysize;
  osivx_int imlen;
  osivx_int firstf;
  osivx_int skipf;
  osivx_int lastf;
  char* name;
  void* file;

  osivx_err* err_info;
} osivx_movie_t;


/* make osivx_preproc_t type */
typedef struct
{
  osivx_int type;
  osivx_int xsize;
  osivx_int ysize;
  osivx_int imlen;
  void * data;

  osivx_movie_t * movie;
} osivx_preproc_t;


/* make osivx_opts_t type */
typedef struct 
{ 
  osivx_int image_xsize; 
  osivx_int image_ysize; 
  osivx_int image_len; 

  /* TODO: remove these once switch
   * to grid engine is finished */
  osivx_int grid_xstart; 
  osivx_int grid_ystart; 
  osivx_int grid_xspacing; 
  osivx_int grid_yspacing; 
  osivx_int grid_xsize; 
  osivx_int grid_ysize; 
  osivx_int grid_len;

  /* TODO: remove these once switch
   * to grid engine is finished */
  osivx_int wind_xsize;
  osivx_int wind_ysize; 
  osivx_int wind_len;
  osivx_int wind_xsuper; 
  osivx_int wind_ysuper; 
  osivx_int wind_slen;

  osivx_int disp_xoffset; 
  osivx_int disp_yoffset; 
  osivx_int disp_xmax; 
  osivx_int disp_ymax; 
  osivx_int disp_xiter;
  osivx_int disp_yiter;

  osivx_int map_xsize;
  osivx_int map_ysize;
  osivx_int map_len;


  /* TODO: remove these once switch
   * to grid engine is finished */
  osivx_int dom_xsize;
  osivx_int dom_ysize;
  osivx_int dom_len;

  /* TODO: remove these once
   * switch to grid engine is
   * finished
   */
  osivx_int fft_xsize;
  osivx_int fft_ysize;
  osivx_int fft_len;
  osivx_int fft_xsuper;
  osivx_int fft_ysuper;
  osivx_int fft_slen;

  osivx_int movie_start; 
  osivx_int movie_finish; 
  osivx_int movie_skip; 
  osivx_int movie_set; 
  osivx_int num_iter; 
  osivx_int num_thrd; 
  osivx_int write_map; 

  osivx_grid_t* grid_data; 
  osivx_interp_t* interp_data;

  osivx_movie_t* movie_afile;
  osivx_preproc_t* preproc_adata;

  osivx_movie_t* movie_bfile;
  osivx_preproc_t* preproc_bdata;

  osivx_file_t* vectors_file;
  osivx_file_t* porthole_file;

  osivx_err* err_info;
} osivx_opts_t; 


/* make osivx_seqen_alg_t type */

typedef struct
{
   void* udata1;
   void* udata2;
   void* udata3;
   void* udata4;
   void* udata5;
} osivx_seqen_alg_t;


/* make osivx_frame_alg_t type */
typedef struct
{
   void* udata1;
   void* udata2;
   void* udata3;
   void* udata4;
   void* udata5;
} osivx_frame_alg_t;


/* make osivx_wind_alg_t type */
typedef struct _osivx_wind_alg_t
{
   osivx_int (*do_glbl_init) (struct _osivx_wind_alg_t* alg,
      const osivx_opts_t* p);

   osivx_int (*do_init) (struct _osivx_wind_alg_t* alg,
      const osivx_opts_t* p, const struct
      _osivx_wind_alg_t* master);

   osivx_int (*do_window) (struct _osivx_wind_alg_t* alg,
      const osivx_opts_t* p,  osivx_grid_iter_t* iter,
      const osivx_imcp_t* im0, const osivx_imcp_t* im1,
      osivx_ioreal_t* std_vec, osivx_ioreal_t* std_map);

   void (*do_free) (struct _osivx_wind_alg_t* alg);

   void (*do_glbl_free) (struct _osivx_wind_alg_t* alg);

   void* udata[10];
} osivx_wind_alg_t;


/* make handing error info macros */
#define OSIVX_ERR_SETNONE(EI)       \
do {                               \
  (EI)->type = OSIVX_ERR_NONE;      \
} while (0)
#define OSIVX_ERR_SETSTR(EI, STR)   \
do {                               \
  (EI)->type = OSIVX_ERR_STRING;    \
  snprintf((EI)->x.str.data,       \
   (EI)->x.str.len, "%s", (STR));  \
} while (0) 
#define OSIVX_ERR_SETINT(EI, ITG)   \
do {                               \
  (EI)->type = OSIVX_ERR_INTEG;     \
  (EI)->x.integ = (ITG);           \
} while (0) 


/* define interpolation neighborhoods */
enum {
  OSIVX_INTERP_NONE,
  OSIVX_INTERP_FOUR,
  OSIVX_INTERP_EIGHT
};


/* extrema.c */
osivx_int osivx_find_min_imcp(osivx_imcp_t*, osivx_int);
osivx_int osivx_find_min_imsp(osivx_imsp_t*, osivx_int);
osivx_int osivx_find_min_immp(osivx_immp_t*, osivx_int);
osivx_int osivx_find_min_imcw(osivx_imcw_t*, osivx_int);
osivx_int osivx_find_min_imsw(osivx_imsw_t*, osivx_int);
osivx_int osivx_find_min_immw(osivx_immw_t*, osivx_int);
osivx_int osivx_find_min_imcf(osivx_imcf_t*, osivx_int);
osivx_int osivx_find_min_imsf(osivx_imsf_t*, osivx_int);
osivx_int osivx_find_min_immf(osivx_immf_t*, osivx_int);
osivx_int osivx_find_min_fftw(fftw_real*, osivx_int,
  osivx_int, osivx_int, osivx_int, osivx_int);

osivx_int osivx_find_max_imcp(osivx_imcp_t*, osivx_int);
osivx_int osivx_find_max_imsp(osivx_imsp_t*, osivx_int);
osivx_int osivx_find_max_immp(osivx_immp_t*, osivx_int);
osivx_int osivx_find_max_imcw(osivx_imcw_t*, osivx_int);
osivx_int osivx_find_max_imsw(osivx_imsw_t*, osivx_int);
osivx_int osivx_find_max_immw(osivx_immw_t*, osivx_int);
osivx_int osivx_find_max_imcf(osivx_imcf_t*, osivx_int);
osivx_int osivx_find_max_imsf(osivx_imsf_t*, osivx_int);
osivx_int osivx_find_max_immf(osivx_immf_t*, osivx_int);
osivx_int osivx_find_max_fftw(fftw_real*, osivx_int,
  osivx_int, osivx_int, osivx_int, osivx_int);


/* generic.c */
osivx_int osivx_generic_wind_alg(osivx_wind_alg_t* alg,
   osivx_opts_t* p);


/* grid.c */
osivx_grid_t* osivx_grid_init_rect(osivx_int, osivx_int, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_int, osivx_int,
   osivx_int, osivx_int);
osivx_int osivx_grid_free(osivx_grid_t*);

void osivx_grid_begin(const osivx_grid_t*, osivx_grid_iter_t*);
osivx_int osivx_grid_iter_at_end(const osivx_grid_iter_t*);
osivx_int osivx_grid_iter_get_info(const osivx_grid_iter_t*,
  osivx_int*, osivx_int*, osivx_int*, osivx_int*,
  osivx_int*, osivx_int*);
osivx_int osivx_grid_iter_next(osivx_grid_iter_t*);

osivx_int osivx_grid_get_len(const osivx_grid_t*);
osivx_int osivx_grid_get_max_wind_sizes(const osivx_grid_t*,
   osivx_int*, osivx_int*, osivx_int*, osivx_int*);
osivx_int osivx_grid_get_mode_wind_sizes(const osivx_grid_t*,
   osivx_int*, osivx_int*, osivx_int*, osivx_int*);
osivx_int osivx_grid_get_range(const osivx_grid_t*, osivx_int*,
  osivx_int*, osivx_int*, osivx_int*);


/* interp.c */

osivx_interp_t* osivx_interp_init(osivx_int);
osivx_int osivx_interp_free(osivx_interp_t*);
osivx_int osivx_interp_usage(const osivx_interp_t*);

void osivx_interp_min_imcp(osivx_interp_t*, osivx_imcp_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_imsp(osivx_interp_t*, osivx_imsp_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_immp(osivx_interp_t*, osivx_immp_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_imcw(osivx_interp_t*, osivx_imcw_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_imsw(osivx_interp_t*, osivx_imsw_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_immw(osivx_interp_t*, osivx_immw_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_imcf(osivx_interp_t*, osivx_imcf_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_imsf(osivx_interp_t*, osivx_imsf_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_immf(osivx_interp_t*, osivx_immf_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_real(osivx_interp_t*, osivx_real*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_min_fftw(osivx_interp_t*, fftw_real*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);

void osivx_interp_max_imcp(osivx_interp_t*, osivx_imcp_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_imsp(osivx_interp_t*, osivx_imsp_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_immp(osivx_interp_t*, osivx_immp_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_imcw(osivx_interp_t*, osivx_imcw_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_imsw(osivx_interp_t*, osivx_imsw_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_immw(osivx_interp_t*, osivx_immw_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_imcf(osivx_interp_t*, osivx_imcf_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_imsf(osivx_interp_t*, osivx_imsf_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_immf(osivx_interp_t*, osivx_immf_t*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_real(osivx_interp_t*, osivx_real*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);
void osivx_interp_max_fftw(osivx_interp_t*, fftw_real*, osivx_int,
   osivx_int, osivx_int, osivx_int, osivx_real*, osivx_real*);


/* movie.c */
osivx_movie_t* osivx_movie_open(const char*, osivx_int,
  osivx_int, osivx_int, osivx_int, osivx_int, osivx_err*);
osivx_int osivx_movie_read(osivx_movie_t*, osivx_int, osivx_imcp_t*);
osivx_int osivx_movie_close(osivx_movie_t*);

/* preproc.c */
osivx_preproc_t* osivx_preproc_init(osivx_movie_t*, osivx_int);
osivx_int osivx_preproc_exec(osivx_preproc_t*, osivx_int, osivx_imcp_t*);
osivx_int osivx_preproc_free(osivx_preproc_t*);

/* alg1.c */
osivx_int osivx_alg1_checkargs(osivx_args_t*);
void osivx_alg1_sumwind(const osivx_imcp_t*, const osivx_imcp_t*,
   osivx_int, osivx_int, osivx_int, osivx_immw_t*);
void osivx_alg1_make(osivx_wind_alg_t*);

/* alg2.c */
osivx_int osivx_alg2_checkargs(osivx_args_t*);
osivx_int osivx_alg2(osivx_opts_t*);

/* alg3.c */
osivx_int osivx_alg3_checkargs(osivx_args_t*);
void osivx_alg3_make(osivx_wind_alg_t*);

/* alg4.c */
osivx_int osivx_alg4_checkargs(osivx_args_t*);
osivx_int osivx_alg4(osivx_opts_t*);

/* alg5.c */
osivx_int osivx_alg5_checkargs(osivx_args_t*);
void osivx_alg5_copy_wind0(const osivx_imcp_t*, osivx_int,
    fftw_real*, osivx_int, osivx_int, osivx_int, osivx_int,
    osivx_int, osivx_int, osivx_int);
void osivx_alg5_copy_wind1(const osivx_imcp_t*, osivx_int,
   fftw_real*, osivx_int, osivx_int, osivx_int, osivx_int,
   osivx_int, osivx_int, osivx_int);
void osivx_alg5_getcorr(fftw_real*, fftw_real*, osivx_int);
void osivx_alg5_map2std(osivx_ioreal_t*, fftw_real*,
   osivx_int, osivx_int, osivx_int, osivx_int, osivx_int);
void osivx_alg5_make(osivx_wind_alg_t*);

/* alg6.c */
osivx_int osivx_alg6_checkargs(osivx_args_t*);
void osivx_alg6_make(osivx_wind_alg_t*);

/* alg7.c */
osivx_int osivx_alg7_checkargs(osivx_args_t*);
void osivx_alg7_make(osivx_wind_alg_t*);

#endif 
