/* FILE: zargs.h
 * 
 * Copyright (c) 2004-2005 James A. Strother 
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 * 
 */


/*
 *  Preprocessing information for zfile functions
 ***************************************************
 */


/*
 * The following are errors returned
 * from the function zfile_parse
 *
 * ZFILE_EOPEN - unable to open file
 * ZFILE_EBUFF - line exceeds buffer
 * ZFILE_ELOST - unrecognized variable
 */
#define ZFILE_EOPEN (-1)
#define ZFILE_EBUFF (-2)
#define ZFILE_ELOST (-3)
#define ZFILE_EJUNK (-4)
#define ZFILE_EFORM (-5)

/*
 * The following are types definitions
 * and public function prototypes
*/

typedef struct
{
   const char* var;
   unsigned evt; 
} zfile_item;


typedef void (*zfile_callb) (long int,
   const char*, const char*, void*);

int zfile_verify(zfile_item*, unsigned);
int zfile_parse(const char*, zfile_item*, 
   unsigned, zfile_callb, void*);


/*
 * Preprocessing information for zopts functions
 ***************************************************
*/


/*
 * The following are option types
 * for passing to zopts_parse
 *
 * ZOPTS_NARG - no argument
 * ZOPTS_WARG - with argument
 * ZOPTS_XARG - optional argument
 */

enum
{
  ZOPTS_NARG,
  ZOPTS_WARG,
  ZOPTS_XARG
};


/*
 * The following are errors which
 * are returned by zopts_parse
 *
 * ZOPTS_ENEED - needs an argument
 * ZOPTS_EWONT - won't take argument
 * ZOPTS_ELOST - option not found
 * ZOPTS_EBUFF - buffer overflow
 */
#define ZOPTS_ENEED (-1)
#define ZOPTS_EWONT (-2)
#define ZOPTS_ELOST (-3)
#define ZOPTS_EBUFF (-4)


/*
 * The following are types definitions
 * and public function prototypes
 */
typedef struct
{
  const char* opt;
  unsigned type;
  unsigned evt;
} zopts_item;


typedef void(*zopts_callb)(long int,
  const char*, const char*, void*);

int zopts_verify(zopts_item*, unsigned);
int zopts_parse(int, char*[],
   zopts_item*, unsigned,
   zopts_item*, unsigned,
   zopts_callb, void*);

